<?php

namespace Sabberworm\CSS\Property;

/**
 * Class representing an @charset rule.
 * The following restrictions apply:
 * • May not be found in any CSSList other than the Document.
 * • May only appear at the very top of a Document’s contents.
 * • Must not appear more than once.
 */
class Charset implements AtRule {

    private $sCharset;

    public function __construct($sCharset) {
        $this->sCharset = $sCharset;
    }

    public function setCharset($sCharset) {
        $this->sCharset = $sCharset;
    }

    public function getCharset() {
        return $this->sCharset;
    }

    public function __toString() {
        return "@charset {$this->sCharset->__toString()};";
    }

    public function atRuleName() {
        return 'charset';
    }

    public function atRuleArgs() {
        return $this->sCharset;
    }
}