<?php

namespace Sabberworm\CSS\Value;

class CSSFunction extends ValueList {

    private $sName;

    public function __construct($sName, $aArguments, $sSeparator = ',') {
        if($aArguments instanceof RuleValueList) {
            $sSeparator = $aArguments->getListSeparator();
            $aArguments = $aArguments->getListComponents();
        }
        $this->sName = $sName;
        parent::__construct($aArguments, $sSeparator);
    }

    public function getName() {
        return $this->sName;
    }

    public function setName($sName) {
        $this->sName = $sName;
    }

    public function getArguments() {
        return $this->aComponents;
    }

    public function __toString() {
        $aArguments = parent::__toString();
        return "{$this->sName}({$aArguments})";
    }

}